@@ -6,8 +6,8 @@ import socket
 import time  # noqa: F401
 import gc  # noqa: F401
 
-from threading import Lock, Condition
-from rpyc.lib import spawn, Timeout, get_methods, get_id_pack
+from threading import Lock, Condition, RLock
+from rpyc.lib import spawn, Timeout, get_methods, get_id_pack, hasattr_static
 from rpyc.lib.compat import pickle, next, maxint, select_error, acquire_lock  # noqa: F401
 from rpyc.lib.colls import WeakValueDict, RefCountingColl
 from rpyc.core import consts, brine, vinegar, netref
@@ -39,7 +39,7 @@ DEFAULT_CONFIG = dict(
                     '__rpow__', '__rrshift__', '__rshift__', '__rsub__', '__rtruediv__',
                     '__rxor__', '__setitem__', '__setslice__', '__str__', '__sub__',
                     '__truediv__', '__xor__', 'next', '__length_hint__', '__enter__',
-                    '__exit__', '__next__', ]),
+                    '__exit__', '__next__', '__format__']),
     exposed_prefix="exposed_",
     allow_getattr=True,
     allow_setattr=False,
@@ -109,7 +109,7 @@ Parameter                                Default value     Description
 
 ``connid``                               ``None``          **Runtime**: the RPyC connection ID (used
                                                            mainly for debugging purposes)
-``credentials``                          ``None``          **Runtime**: the credentails object that was returned
+``credentials``                          ``None``          **Runtime**: the credentials object that was returned
                                                            by the server's :ref:`authenticator <api-authenticators>`
                                                            or ``None``
 ``endpoints``                            ``None``          **Runtime**: The connection's endpoints. This is a tuple
@@ -140,14 +140,14 @@ class Connection(object):
         self._config = DEFAULT_CONFIG.copy()
         self._config.update(config)
         if self._config["connid"] is None:
-            self._config["connid"] = "conn%d" % (next(_connection_id_generator),)
+            self._config["connid"] = f"conn{next(_connection_id_generator)}"
 
         self._HANDLERS = self._request_handlers()
         self._channel = channel
         self._seqcounter = itertools.count()
-        self._recvlock = Lock()
+        self._recvlock = RLock()  # AsyncResult implementation means that synchronous requests have multiple acquires
         self._sendlock = Lock()
-        self._recv_event = Condition()
+        self._recv_event = Condition()  # TODO: why not simply timeout? why not associate w/ recvlock? explain/redesign
         self._request_callbacks = {}
         self._local_objects = RefCountingColl()
         self._last_traceback = None
@@ -169,7 +169,7 @@ class Connection(object):
 
     def __repr__(self):
         a, b = object.__repr__(self).split(" object ")
-        return "%s %r object %s" % (a, self._config["connid"], b)
+        return f"{a} {self._config['connid']!r} object {b}"
 
     def _cleanup(self, _anyway=True):  # IO
         if self._closed and not _anyway:
@@ -298,7 +298,7 @@ class Connection(object):
                 proxy = self._netref_factory(id_pack)
                 self._proxy_cache[id_pack] = proxy
             return proxy
-        raise ValueError("invalid label %r" % (label,))
+        raise ValueError(f"invalid label {label!r}")
 
     def _netref_factory(self, id_pack):  # boxing
         """id_pack is for remote, so when class id fails to directly match """
@@ -367,7 +367,7 @@ class Connection(object):
             obj = self._unbox_exc(args)
             self._seq_request_callback(msg, seq, True, obj)
         else:
-            raise ValueError("invalid message type: %r" % (msg,))
+            raise ValueError(f"invalid message type: {msg!r}")
 
     def serve(self, timeout=1, wait_for_lock=True):  # serving
         """Serves a single request or reply that arrives within the given
@@ -380,21 +380,35 @@ class Connection(object):
         """
         timeout = Timeout(timeout)
         with self._recv_event:
+            # Exit early if we cannot acquire the recvlock
             if not self._recvlock.acquire(False):
-                return wait_for_lock and self._recv_event.wait(timeout.timeleft())
+                if wait_for_lock:
+                    # Wait condition for recvlock release; recvlock is not underlying lock for condition
+                    return self._recv_event.wait(timeout.timeleft())
+                else:
+                    return False
+        # Assume the receive rlock is acquired and incremented
         try:
+            data = None  # Ensure data is initialized
             data = self._channel.poll(timeout) and self._channel.recv()
-            if not data:
-                return False
-        except EOFError:
-            self.close()
-            raise
-        finally:
+        except Exception as exc:
+            if isinstance(exc, EOFError):
+                self.close()  # sends close async request
             self._recvlock.release()
             with self._recv_event:
                 self._recv_event.notify_all()
-        self._dispatch(data)
-        return True
+            raise
+        # At this point, the recvlock was acquired once, we must release once before exiting the function
+        if data:
+            # Dispatch will unbox, invoke callbacks, etc.
+            self._dispatch(data)
+            self._recvlock.release()
+            with self._recv_event:
+                self._recv_event.notify_all()
+            return True
+        else:
+            self._recvlock.release()
+            return False
 
     def poll(self, timeout=0):  # serving
         """Serves a single transaction, should one arrives in the given
@@ -463,14 +477,17 @@ class Connection(object):
             pass
         return at_least_once
 
-    def sync_request(self, handler, *args):  # serving
+    def sync_request(self, handler, *args):
         """requests, sends a synchronous request (waits for the reply to arrive)
 
         :raises: any exception that the requets may be generated
         :returns: the result of the request
         """
         timeout = self._config["sync_request_timeout"]
-        return self.async_request(handler, *args, timeout=timeout).value
+        _async_res = self.async_request(handler, *args, timeout=timeout)
+        # _async_res is an instance of AsyncResult, the value property invokes Connection.serve via AsyncResult.wait
+        # So, the _recvlock can be acquired multiple times by the owning thread and warrants the use of RLock
+        return _async_res.value
 
     def _async_request(self, handler, args=(), callback=(lambda a, b: None)):  # serving
         seq = self._get_seq_id()
@@ -492,7 +509,7 @@ class Connection(object):
         """
         timeout = kwargs.pop("timeout", None)
         if kwargs:
-            raise TypeError("got unexpected keyword argument(s) %s" % (list(kwargs.keys()),))
+            raise TypeError("got unexpected keyword argument(s) {list(kwargs.keys()}")
         res = AsyncResult(self)
         self._async_request(handler, args, res)
         if timeout is not None:
@@ -509,20 +526,20 @@ class Connection(object):
     def _check_attr(self, obj, name, perm):  # attribute access
         config = self._config
         if not config[perm]:
-            raise AttributeError("cannot access %r" % (name,))
+            raise AttributeError(f"cannot access {name!r}")
         prefix = config["allow_exposed_attrs"] and config["exposed_prefix"]
         plain = config["allow_all_attrs"]
         plain |= config["allow_exposed_attrs"] and name.startswith(prefix)
         plain |= config["allow_safe_attrs"] and name in config["safe_attrs"]
         plain |= config["allow_public_attrs"] and not name.startswith("_")
-        has_exposed = prefix and hasattr(obj, prefix + name)
+        has_exposed = prefix and (hasattr(obj, prefix + name) or hasattr_static(obj, prefix + name))
         if plain and (not has_exposed or hasattr(obj, name)):
             return name
         if has_exposed:
             return prefix + name
         if plain:
             return name  # chance for better traceback
-        raise AttributeError("cannot access %r" % (name,))
+        raise AttributeError(f"cannot access {name!r}")
 
     def _access_attr(self, obj, name, args, overrider, param, default):  # attribute access
         if type(name) is bytes:
