@@ -19,6 +19,7 @@ if not isinstance(MyMeta, MyMeta):
     MyClass = MyMeta(MyClass.__name__, MyClass.__bases__, dict(MyClass.__dict__))
 
 
+@rpyc.service
 class MyService(rpyc.Service):
     on_connect_called = False
     on_disconnect_called = False
@@ -50,6 +51,24 @@ class MyService(rpyc.Service):
     def exposed_instance(self, inst, cls):
         return isinstance(inst, cls)
 
+    @rpyc.exposed
+    class MyClass(object):
+        def __init__(self, a, b):
+            self.a = a
+            self.b = b
+
+        @rpyc.exposed
+        def foo(self):
+            return self.a + self.b
+
+    @rpyc.exposed
+    def get_decorated(self):
+        return "decorated"
+
+    @rpyc.exposed('prefix_')
+    def get_decorated_prefix(self):
+        return "decorated_prefix"
+
 
 def before_closed(root):
     root.on_about_to_close()
@@ -61,9 +80,13 @@ class TestCustomService(unittest.TestCase):
     def setUp(self):
         self.service = MyService()
         client_config = {"before_closed": before_closed, "close_catchall": False}
-        self.conn = rpyc.connect_thread( remote_service=self.service, config=client_config)
-
+        prefixed_client_config = {'exposed_prefix': 'prefix_'}
+        self.conn = rpyc.connect_thread(remote_service=self.service, config=client_config)
+        self.prefixed_conn = rpyc.connect_thread(remote_service=self.service,
+                                                 config=prefixed_client_config,
+                                                 remote_config=prefixed_client_config)
         self.conn.root  # this will block until the service is initialized,
+        self.prefixed_conn.root  # this will block until the service is initialized,
         # so we can be sure on_connect_called is True by that time
         self.assertTrue(self.service.on_connect_called)
 
@@ -80,7 +103,7 @@ class TestCustomService(unittest.TestCase):
         self.assertTrue(self.service.on_about_to_close_called)
 
     def test_aliases(self):
-        print("service name: %s" % (self.conn.root.get_service_name(),))
+        print(f"service name: {self.conn.root.get_service_name()}")
 
     def test_distance(self):
         assert self.conn.root.distance((2, 7), (5, 11)) == 5
@@ -92,6 +115,14 @@ class TestCustomService(unittest.TestCase):
         self.conn.root.exposed_getlist
         # this is not an exposed attribute:
         self.assertRaises(AttributeError, lambda: self.conn.root.foobar())
+        # methods exposed using decorator
+        self.conn.root.get_decorated
+        self.conn.root.exposed_get_decorated
+        self.prefixed_conn.root.get_decorated_prefix
+        self.prefixed_conn.root.prefix_get_decorated_prefix
+        self.assertFalse(hasattr(self.conn.root, 'get_decorated_prefix'))
+        smc = self.conn.root.MyClass('a', 'b')
+        self.assertEquals(smc.foo(), 'ab')
 
     def test_safeattrs(self):
         x = self.conn.root.getlist()
